@@ -15,6 +15,7 @@
 package register
 
 import (
+	_ "github.com/coreos/ignition/v2/internal/providers/akamai"
 	_ "github.com/coreos/ignition/v2/internal/providers/aliyun"
 	_ "github.com/coreos/ignition/v2/internal/providers/applehv"
 	_ "github.com/coreos/ignition/v2/internal/providers/aws"
@@ -35,6 +36,7 @@ import (
 	_ "github.com/coreos/ignition/v2/internal/providers/packet"
 	_ "github.com/coreos/ignition/v2/internal/providers/powervs"
 	_ "github.com/coreos/ignition/v2/internal/providers/qemu"
+	_ "github.com/coreos/ignition/v2/internal/providers/scaleway"
 	_ "github.com/coreos/ignition/v2/internal/providers/virtualbox"
 	_ "github.com/coreos/ignition/v2/internal/providers/vmware"
 	_ "github.com/coreos/ignition/v2/internal/providers/vultr"
